// UI.CPP


#include "src/bsuite.h"
#include <dos.h>
#include <string.h>
#include <conio.h>


char far *screen = (char far *)MK_FP(0xB800,0);
int far *wscreen = (int far *)MK_FP(0xB800,0);


#define LINE_SINGLE_UL	''
#define LINE_SINGLE_UR	''
#define LINE_SINGLE_LL	''
#define LINE_SINGLE_LR	''
#define LINE_SINGLE_HZ	''
#define LINE_SINGLE_VT	''

#define LINE_DOUBLE_UL	''
#define LINE_DOUBLE_UR	''
#define LINE_DOUBLE_LL	''
#define LINE_DOUBLE_LR	''
#define LINE_DOUBLE_HZ	''
#define LINE_DOUBLE_VT	''


/////////////////////////////////////////////////
//
// Screen output functions

void uiPutChar(int x, int y, const char c, char fg, char bg)
{
	if ((x>SCREEN_WIDTH) || (x<1)) return;
	if ((y>SCREEN_DEPTH) || (y<1)) return;

	unsigned of = (y-1) * (SCREEN_WIDTH * 2) + (x-1) * 2;

	screen[of]   = c;
	screen[of+1] = (bg<<4)+fg;
}

void uiPrintString(int x, int y, const char *s, char fg, char bg)
{
	if ((x>SCREEN_WIDTH) || (x<1)) return;
	if ((y>SCREEN_DEPTH) || (y<1)) return;

	unsigned of = (y-1) * (SCREEN_WIDTH * 2) + (x-1) * 2;
	unsigned lp, cc=0, len=strlen(s);
	char cfg=fg, cbg=bg;


	for (lp=0; lp<len; lp++)
	{
		if (s[lp] == '~')	// in-string control code
		{
			lp++;	// skip the marker

			if (s[lp] == '@')			// revert to normal colours
			{
				cfg=fg; cbg=bg;

			} else if (s[lp] == '!') {	// change colour code
				lp++;	// skip the '!'

				// foreground first
				if ((s[lp] >= '0') && (s[lp] <= '9'))
					cfg = s[lp] - '0';
				else if ((s[lp] >= 'A') && (s[lp] <= 'F'))
					cfg = s[lp] - 'A' + 10;

				lp++;

				// background next
				if ((s[lp] >= '0') && (s[lp] <= '9'))
					cbg = s[lp] - '0';
				else if ((s[lp] >= 'A') && (s[lp] <= 'F'))
					cbg = s[lp] - 'A' + 10;

			} else if (s[lp] == '~') {	// print a '~'
				screen[of+(cc<<1)]   = s[lp];
				screen[of+(cc<<1)+1] = (cbg<<4)+cfg;
				cc++;
			}
		} else {
			screen[of+(cc<<1)]   = s[lp];
			screen[of+(cc<<1)+1] = (cbg<<4)+cfg;
			cc++;
		}
	}
}

/////////////////////////////////////////////////
//
// Screen clearing

void uiClearScreen(char bg)
{
	unsigned fb = (unsigned)0x20 | ((unsigned)bg << 12) | ((unsigned)7<<8), lp;

	for (lp=0; lp < (SCREEN_WIDTH*SCREEN_DEPTH); lp++)
		wscreen[lp] = fb;
}

void uiClearRegion(int ox, int oy, char w, char d, char bg, char fg)
{
	int lp, lq;

	for (lp=0; lp<w; lp++)
	{
		for (lq=0; lq<d; lq++)
		{
			uiPutChar(ox+lp, oy+lq, 0x20, fg, bg);
		}
	}
}

/////////////////////////////////////////////////
//
// Primitive drawing

void uiDrawSingleBox(int ox, int oy, char w, char d, char fg, char bg)
{
	int lp, lq;

	uiPutChar(ox,       oy,       LINE_SINGLE_UL, fg, bg);
	uiPutChar(ox+(w-1), oy,       LINE_SINGLE_UR, fg, bg);
	uiPutChar(ox,       oy+(d-1), LINE_SINGLE_LL, fg, bg);
	uiPutChar(ox+(w-1), oy+(d-1), LINE_SINGLE_LR, fg, bg);

	for (lp=0; lp<(w-2); lp++)
	{
		uiPutChar(ox+1+lp, oy,       LINE_SINGLE_HZ, fg, bg);
		uiPutChar(ox+1+lp, oy+(d-1), LINE_SINGLE_HZ, fg, bg);
	}

	for (lp=0; lp<(d-2); lp++)
	{
		uiPutChar(ox, oy+1+lp,       LINE_SINGLE_VT, fg, bg);
		uiPutChar(ox+(w-1), oy+1+lp, LINE_SINGLE_VT, fg, bg);
	}

	for (lp=0; lp<(w-2); lp++)
	{
		for (lq=0; lq<(d-2); lq++)
		{
			uiPutChar(ox+1+lp, oy+1+lq, 0x20, fg, bg);
		}
	}
}

void uiDrawDoubleBox(int ox, int oy, char w, char d, char fg, char bg)
{
	int lp, lq;

	uiPutChar(ox,       oy,       LINE_DOUBLE_UL, fg, bg);
	uiPutChar(ox+(w-1), oy,       LINE_DOUBLE_UR, fg, bg);
	uiPutChar(ox,       oy+(d-1), LINE_DOUBLE_LL, fg, bg);
	uiPutChar(ox+(w-1), oy+(d-1), LINE_DOUBLE_LR, fg, bg);

	for (lp=0; lp<(w-2); lp++)
	{
		uiPutChar(ox+1+lp, oy,       LINE_DOUBLE_HZ, fg, bg);
		uiPutChar(ox+1+lp, oy+(d-1), LINE_DOUBLE_HZ, fg, bg);
	}

	for (lp=0; lp<(d-2); lp++)
	{
		uiPutChar(ox, oy+1+lp,       LINE_DOUBLE_VT, fg, bg);
		uiPutChar(ox+(w-1), oy+1+lp, LINE_DOUBLE_VT, fg, bg);
	}

	for (lp=0; lp<(w-2); lp++)
	{
		for (lq=0; lq<(d-2); lq++)
		{
			uiPutChar(ox+1+lp, oy+1+lq, 0x20, fg, bg);
		}
	}
}

/////////////////////////////////////////////////
//
// Simple UI devices

int uiMessage(const char *msg)
{
	char buf[80], copied;
	const char *pmsg;
	int strs=1, longest=0, len=0, lp;
	int width, height, ox, oy;

	// count the number of lines in the message
	for (lp=0; lp<strlen(msg); lp++)
	{
		if (msg[lp] == '\n')
		{
			strs++;
			if (len > longest)
				longest = len;
			len=0;
		} else {
			len++;
		}
	}

	if (strs == 1) longest = len;	// if only 1 line then longest = len msg

	ox = (int)(SCREEN_WIDTH/2 - longest/2 - 2);
	oy = (int)(SCREEN_DEPTH/2 - strs/2 - 1);
	width  = longest + 5;
	height = strs + 2;

	uiDrawDoubleBox(ox, oy, width, height, 11, 3);

	// print the strings
	pmsg = msg;
	for (lp=0; lp<strs; lp++)
	{
		copied=0;
		while ((*pmsg != '\n') && (*pmsg != 0))
		{
			buf[copied++] = *pmsg++;
		}

		pmsg++;
		buf[copied] = 0;

		uiPrintString(ox + 2, oy + 1 + lp, buf, 15, 3);
	}

	getch();		// wait for keypress
	while (kbhit()) getch();

	uiClearRegion(ox, oy, width, height, DESK_BACK_COLOUR);	// erase the window

	return 1;
}


#define BACKSPACE 8
#define ENTER 		13
#define ESCAPE		27

int uiInput(const char *msg, char *data, int maxlen)
{
	char buf[80], copied;
	const char *pmsg;
	int strs=1, longest=maxlen, len=0, lp;
	int width, height, ox, oy;
	int key=-1;

	// count the number of lines in the message
	for (lp=0; lp<strlen(msg); lp++)
	{
		if (msg[lp] == '\n')
		{
			strs++;
			if (len > longest)
				longest = len;
			len=0;
		} else {
			len++;
		}
	}

	if (strs == 1)
		if (len > longest)
			longest = len;	// only 1 line

	ox = (int)(SCREEN_WIDTH/2 - longest/2 - 2);
	oy = (int)(SCREEN_DEPTH/2 - strs/2 - 1);
	width  = longest + 5;
	height = strs + 3;

	uiDrawDoubleBox(ox, oy, width, height, 11, 3);

	// print the strings
	pmsg = msg;
	for (lp=0; lp<strs; lp++)
	{
		copied=0;
		while ((*pmsg != '\n') && (*pmsg != 0))
		{
			buf[copied++] = *pmsg++;
		}

		pmsg++;
		buf[copied] = 0;

		uiPrintString(ox + 2, oy + 1 + lp, buf, 15, 3);
	}

	// input the data
	uiClearRegion(ox+2, oy+strs+1, maxlen+1, 1, 7, 0);
	uiPrintString(ox+2, oy+strs+1, data, 0, 7);

	len = strlen(data);
	while ((key != ESCAPE) && (key != ENTER))
	{
		key = getch();

		if ((key == ESCAPE) || (key == ENTER)) break;	// escape & enter
		else if (key == 0) {	// extended key (1st byte==0, 2nd=scancode)
			getch();
		} else if (key == BACKSPACE) {	// backspace
			if (len > 0)
			{
				data[--len] = 0;

				uiClearRegion(ox+2, oy+strs+1, maxlen+1, 1, 7, 0);
				uiPrintString(ox+2, oy+strs+1, data, 0, 7);
			}
		} else if ((key >= 32) && (key <= 127)) {		// normal character
			if (len < maxlen)
			{
				data[len++] = key;
				data[len]   = 0;

				uiClearRegion(ox+2, oy+strs+1, maxlen+1, 1, 7, 0);
				uiPrintString(ox+2, oy+strs+1, data, 0, 7);
			}
		}
	}

	uiClearRegion(ox, oy, width, height, DESK_BACK_COLOUR);	// erase the window

	if (key == ESCAPE) return 1;
	else return 0;
}